import { StatusCodeError } from "request-promise-native/errors";
import Helper from "../utils/helper";
import Problem from "./problem";
class Leetcode {
    session;
    csrfToken;
    constructor(credit) {
        this.session = credit.session;
        this.csrfToken = credit.csrfToken;
    }
    get credit() {
        return {
            session: this.session,
            csrfToken: this.csrfToken,
        };
    }
    static async build(endpoint, props) {
        Helper.switchEndPoint(endpoint);
        const credit = await (async () => {
            if (props?.cookie) {
                return await this.getCredit(props.cookie.split(" "));
            }
            return await this.login(props?.username ?? "", props?.password ?? "");
        })();
        Helper.setCredit(credit);
        return new Leetcode(credit);
    }
    static async login(username, password) {
        const csrfToken = await (async () => {
            // got login token first
            const response = await Helper.HttpRequest({
                url: Helper.uris.login,
                resolveWithFullResponse: true,
            });
            // Leetcode CN return null here, but it's does not matter
            const csrfToken = Helper.parseCookie(response.headers["set-cookie"], "csrftoken");
            Helper.setCredit({ csrfToken });
            return csrfToken;
        })();
        try {
            const response = await Helper.HttpRequest({
                method: "POST",
                url: Helper.uris.login,
                form: {
                    csrfmiddlewaretoken: csrfToken,
                    login: username,
                    password: password,
                },
                resolveWithFullResponse: true,
            });
            return await this.getCredit(response.headers["set-cookie"]);
        }
        catch (e) {
            if (e instanceof StatusCodeError) {
                throw new Error("Login Fail");
            }
        }
        return { csrfToken };
    }
    static async getCredit(cookie) {
        const session = Helper.parseCookie(cookie, "LEETCODE_SESSION");
        const csrfToken = Helper.parseCookie(cookie, "csrftoken");
        return {
            session,
            csrfToken,
        };
    }
    async getProfile() {
        // TODO: fetch more user profile.
        const response = await Helper.GraphQLRequest({
            query: `
            {
                user {
                    username
                }
            }
            `,
        });
        return response.user;
    }
    async getAllProblems() {
        let response = await Helper.HttpRequest({
            url: Helper.uris.problemsAll,
        });
        response = JSON.parse(response);
        const problems = response.stat_status_pairs.map((p) => {
            return new Problem(p.stat.question__title_slug, p.stat.question_id, p.stat.frontend_question_id, p.stat.question__title, undefined, Helper.difficultyMap(p.difficulty.level), p.is_favor, p.paid_only, undefined, undefined, Helper.statusMap(p.status), undefined, p.stat.total_acs, p.stat.total_submitted, undefined, undefined, undefined);
        });
        return problems;
    }
    async getProblemsByTag(tag) {
        const response = await Helper.GraphQLRequest({
            query: `
                query getTopicTag($slug: String!) {
                    topicTag(slug: $slug) {
                        questions {
                            status
                            questionId
                            title
                            titleSlug
                            stats
                            difficulty
                            isPaidOnly
                            topicTags {
                                slug
                            }
                        }
                    }
                }
            `,
            variables: {
                slug: tag,
            },
        });
        const problems = response.topicTag.questions.map((p) => {
            const stat = JSON.parse(p.stats);
            return new Problem(p.titleSlug, p.questionId, p.title, stat.title, undefined, p.isPaidOnly, undefined, undefined, Helper.statusMap(p.status), p.topicTags.map((t) => t.slug), stat.totalAcceptedRaw, stat.totalSubmissionRaw, undefined, undefined, undefined);
        });
        return problems;
    }
}
export default Leetcode;
//# sourceMappingURL=leetcode.js.map